@@ -31,7 +31,7 @@
  * The above mentioned pipeline should dump data packets to the console.
  *
  * If the #GstFdSrc:timeout property is set to a value bigger than 0, fdsrc will
- * generate an element message named <classname>&quot;GstFdSrcTimeout&quot;</classname>
+ * generate an element message named `GstFdSrcTimeout`
  * if no data was received in the given timeout.
  *
  * The message's structure contains one field:
@@ -79,12 +79,17 @@
 #include <errno.h>
 
 #include "gstfdsrc.h"
+#include "gstcoreelementselements.h"
+
+#define struct_stat struct stat
 
 #ifdef __BIONIC__               /* Android */
-#undef lseek
-#define lseek lseek64
+#if defined(__ANDROID_API__) && __ANDROID_API__ >= 21
 #undef fstat
 #define fstat fstat64
+#undef struct_stat
+#define struct_stat struct stat64
+#endif
 #endif
 
 static GstStaticPadTemplate srctemplate = GST_STATIC_PAD_TEMPLATE ("src",
@@ -115,6 +120,9 @@ static void gst_fd_src_uri_handler_init (gpointer g_iface, gpointer iface_data);
   GST_DEBUG_CATEGORY_INIT (gst_fd_src_debug, "fdsrc", 0, "fdsrc element");
 #define gst_fd_src_parent_class parent_class
 G_DEFINE_TYPE_WITH_CODE (GstFdSrc, gst_fd_src, GST_TYPE_PUSH_SRC, _do_init);
+#if defined(HAVE_SYS_SOCKET_H) || defined(G_OS_WIN32)
+GST_ELEMENT_REGISTER_DEFINE (fdsrc, "fdsrc", GST_RANK_NONE, GST_TYPE_FD_SRC);
+#endif
 
 static void gst_fd_src_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
@@ -208,7 +216,7 @@ gst_fd_src_dispose (GObject * obj)
 static void
 gst_fd_src_update_fd (GstFdSrc * src, guint64 size)
 {
-  struct stat stat_results;
+  struct_stat stat_results;
 
   GST_DEBUG_OBJECT (src, "fdset %p, old_fd %d, new_fd %d", src->fdset, src->fd,
       src->new_fd);
@@ -388,7 +396,7 @@ gst_fd_src_create (GstPushSrc * psrc, GstBuffer ** outbuf)
   guint blocksize;
   GstMapInfo info;
 
-#ifndef HAVE_WIN32
+#ifndef G_OS_WIN32
   GstClockTime timeout;
   gboolean try_again;
   gint retval;
@@ -396,7 +404,7 @@ gst_fd_src_create (GstPushSrc * psrc, GstBuffer ** outbuf)
 
   src = GST_FD_SRC (psrc);
 
-#ifndef HAVE_WIN32
+#ifndef G_OS_WIN32
   if (src->timeout > 0) {
     timeout = src->timeout * GST_USECOND;
   } else {
@@ -442,10 +450,16 @@ gst_fd_src_create (GstPushSrc * psrc, GstBuffer ** outbuf)
   if (!gst_buffer_map (buf, &info, GST_MAP_WRITE))
     goto buffer_read_error;
 
+#ifdef G_OS_WIN32
+  int cur_mode = _setmode (src->fd, O_BINARY);
+#endif
   do {
     readbytes = read (src->fd, info.data, blocksize);
     GST_LOG_OBJECT (src, "read %" G_GSSIZE_FORMAT, readbytes);
   } while (readbytes == -1 && errno == EINTR);  /* retry if interrupted */
+#ifdef G_OS_WIN32
+  _setmode (src->fd, cur_mode);
+#endif
 
   if (readbytes < 0)
     goto read_error;
@@ -468,7 +482,7 @@ gst_fd_src_create (GstPushSrc * psrc, GstBuffer ** outbuf)
   return GST_FLOW_OK;
 
   /* ERRORS */
-#ifndef HAVE_WIN32
+#ifndef G_OS_WIN32
 poll_error:
   {
     GST_ELEMENT_ERROR (src, RESOURCE, READ, (NULL),
@@ -544,7 +558,7 @@ static gboolean
 gst_fd_src_get_size (GstBaseSrc * bsrc, guint64 * size)
 {
   GstFdSrc *src = GST_FD_SRC (bsrc);
-  struct stat stat_results;
+  struct_stat stat_results;
 
   if (src->size != -1) {
     *size = src->size;
